@@ -37,14 +37,14 @@ namespace KAuth
 static void debugMessageReceived(int t, const QString &message);
 
 DBusHelperProxy::DBusHelperProxy()
-    : responder(0)
+    : responder(nullptr)
     , m_stopRequest(false)
     , m_busConnection(QDBusConnection::systemBus())
 {
 }
 
 DBusHelperProxy::DBusHelperProxy(const QDBusConnection &busConnection)
-    : responder(0)
+    : responder(nullptr)
     , m_stopRequest(false)
     , m_busConnection(busConnection)
 {
@@ -52,11 +52,6 @@ DBusHelperProxy::DBusHelperProxy(const QDBusConnection &busConnection)
 
 DBusHelperProxy::~DBusHelperProxy()
 {
-    // workaround for Qt 5.6.0 bug which leads to kded5 deadlocking when a program using kauth exits
-    // See: https://bugreports.qt.io/browse/QTBUG-51648
-#if QT_VERSION >= QT_VERSION_CHECK(5, 6, 0) && QT_VERSION < QT_VERSION_CHECK(5, 7, 0)
-    disconnect();
-#endif
 }
 
 void DBusHelperProxy::stopAction(const QString &action, const QString &helperID)
@@ -71,7 +66,7 @@ void DBusHelperProxy::stopAction(const QString &action, const QString &helperID)
     m_busConnection.asyncCall(message);
 }
 
-void DBusHelperProxy::executeAction(const QString &action, const QString &helperID, const QVariantMap &arguments)
+void DBusHelperProxy::executeAction(const QString &action, const QString &helperID, const QVariantMap &arguments, int timeout)
 {
     QByteArray blob;
     {
@@ -93,7 +88,10 @@ void DBusHelperProxy::executeAction(const QString &action, const QString &helper
     //if already connected reply will be false but we won't have an error or a reason to fail
     if (!connected && m_busConnection.lastError().isValid()) {
         ActionReply errorReply = ActionReply::DBusErrorReply();
-        errorReply.setErrorDescription(tr("DBus Backend error: connection to helper failed. %1").arg(m_busConnection.lastError().message()));
+        errorReply.setErrorDescription(tr("DBus Backend error: connection to helper failed. %1\n(application: %2 helper: %3)").arg(
+                m_busConnection.lastError().message(),
+                qApp->applicationName(),
+                helperID));
         emit actionPerformed(action, errorReply);
         return;
     }
@@ -107,7 +105,7 @@ void DBusHelperProxy::executeAction(const QString &action, const QString &helper
 
     m_actionsInProgress.push_back(action);
 
-    QDBusPendingCall pendingCall = m_busConnection.asyncCall(message);
+    QDBusPendingCall pendingCall = m_busConnection.asyncCall(message, timeout);
 
     auto watcher = new QDBusPendingCallWatcher(pendingCall, this);
 
@@ -127,51 +125,17 @@ void DBusHelperProxy::executeAction(const QString &action, const QString &helper
     });
 }
 
-Action::AuthStatus DBusHelperProxy::authorizeAction(const QString &action, const QString &helperID)
-{
-    if (!m_actionsInProgress.isEmpty()) {
-        return Action::ErrorStatus;
-    }
-
-    m_busConnection.interface()->startService(helperID);
-
-    QDBusMessage message;
-    message = QDBusMessage::createMethodCall(helperID, QLatin1String("/"), QLatin1String("org.kde.kf5auth"), QLatin1String("authorizeAction"));
-
-    QList<QVariant> args;
-    args << action << BackendsManager::authBackend()->callerID();
-    message.setArguments(args);
-
-    m_actionsInProgress.push_back(action);
-
-    QEventLoop e;
-    QDBusPendingCall pendingCall = m_busConnection.asyncCall(message);
-    QDBusPendingCallWatcher watcher(pendingCall, this);
-    connect(&watcher, SIGNAL(finished(QDBusPendingCallWatcher*)), &e, SLOT(quit()));
-    e.exec();
-
-    m_actionsInProgress.removeOne(action);
-
-    QDBusMessage reply = pendingCall.reply();
-
-    if (reply.type() == QDBusMessage::ErrorMessage || reply.arguments().size() != 1) {
-        return Action::ErrorStatus;
-    }
-
-    return static_cast<Action::AuthStatus>(reply.arguments().first().toUInt());
-}
-
 bool DBusHelperProxy::initHelper(const QString &name)
 {
     new Kf5authAdaptor(this);
 
     if (!m_busConnection.registerService(name)) {
-        qDebug() << "couldn't register service" << m_busConnection.lastError();
+        qCWarning(KAUTH) << "Error registering helper DBus service" << name << m_busConnection.lastError().message();
         return false;
     }
 
     if (!m_busConnection.registerObject(QLatin1String("/"), this)) {
-        qDebug() << "couldn't register object" << m_busConnection.lastError();
+        qCWarning(KAUTH) << "Error registering helper DBus object:" << m_busConnection.lastError().message();
         return false;
     }
 
@@ -232,6 +196,29 @@ bool DBusHelperProxy::hasToStopAction()
     return m_stopRequest;
 }
 
+bool DBusHelperProxy::isCallerAuthorized(const QString &action, const QByteArray &callerID)
+{
+    // Check the caller is really who it says it is
+    switch (BackendsManager::authBackend()->extraCallerIDVerificationMethod()) {
+        case AuthBackend::NoExtraCallerIDVerificationMethod:
+        break;
+
+        case AuthBackend::VerifyAgainstDBusServiceName:
+            if (message().service().toUtf8() != callerID) {
+                return false;
+            }
+        break;
+
+        case AuthBackend::VerifyAgainstDBusServicePid:
+            if (connection().interface()->servicePid(message().service()).value() != callerID.toUInt()) {
+                return false;
+            }
+        break;
+    }
+
+    return BackendsManager::authBackend()->isCallerAuthorized(action, callerID);
+}
+
 QByteArray DBusHelperProxy::performAction(const QString &action, const QByteArray &callerID, QByteArray arguments)
 {
     if (!responder) {
@@ -256,7 +243,7 @@ QByteArray DBusHelperProxy::performAction(const QString &action, const QByteArra
     QTimer *timer = responder->property("__KAuth_Helper_Shutdown_Timer").value<QTimer *>();
     timer->stop();
 
-    if (BackendsManager::authBackend()->isCallerAuthorized(action, callerID)) {
+    if (isCallerAuthorized(action, callerID)) {
         QString slotname = action;
         if (slotname.startsWith(m_name + QLatin1Char('.'))) {
             slotname = slotname.right(slotname.length() - m_name.length() - 1);
@@ -285,31 +272,6 @@ QByteArray DBusHelperProxy::performAction(const QString &action, const QByteArra
     return retVal.serialized();
 }
 
-uint DBusHelperProxy::authorizeAction(const QString &action, const QByteArray &callerID)
-{
-    if (!m_currentAction.isEmpty()) {
-        return static_cast<uint>(Action::ErrorStatus);
-    }
-
-    m_currentAction = action;
-
-    uint retVal;
-
-    QTimer *timer = responder->property("__KAuth_Helper_Shutdown_Timer").value<QTimer *>();
-    timer->stop();
-
-    if (BackendsManager::authBackend()->isCallerAuthorized(action, callerID)) {
-        retVal = static_cast<uint>(Action::AuthorizedStatus);
-    } else {
-        retVal = static_cast<uint>(Action::DeniedStatus);
-    }
-
-    timer->start();
-    m_currentAction.clear();
-
-    return retVal;
-}
-
 void DBusHelperProxy::sendDebugMessage(int level, const char *msg)
 {
     QByteArray blob;
